# Single-omics analysis part, maybe make this one a separate R file

## Data file names
args <- commandArgs(trailingOnly=TRUE)
if(length(args)<4){
    cat("Usage:\n\tR --args omics_training_file omics_validation_file output_folder methods\n");
    cat("The methods option may contain any combinations of the following option(s): randomforest,pca,spca,plsda,splsda\n");
    cat("However, in the case of sparse PCA and sparse PLS-DA, default of 2 components and for each component 100 features would be used\n");
    q(save="no");
}
library(randomForest)
library(mixOmics)
library(ggplot2)
omics_training_file = args[1];
omics_validation_file = args[2];
output_folder= args[3];
methods = unlist(strsplit(args[4],","));


## Read the training file
omics_train_df1 <- read.table(omics_training_file, sep="\t", header=F, row.names=1)
# Remove first 2 rows and use it as a column name and separate column, respectively
omics_train_df <- as.data.frame(t(omics_train_df1[3:nrow(omics_train_df1),]), stringsAsFactors = F); 
omics_train_df <- as.data.frame(cbind(t(omics_train_df1[2,]),omics_train_df))
rownames(omics_train_df) <- as.vector(unlist(omics_train_df1[1,]))
# Since the 2nd row contains response/class/status inormation we want to use that as a response variable
colnames(omics_train_df)[1] <- "response"

## Read the validation file
omics_validation_df1 <- read.table(omics_validation_file, sep="\t", header=F, row.names=1)
# Remove first 2 rows and use it as a column name and separate column, respectively
omics_validation_df <- as.data.frame(t(omics_validation_df1[3:nrow(omics_validation_df1),]),stringsAsFactors = F); 
omics_validation_df <- as.data.frame(cbind(t(omics_validation_df1[2,]),omics_validation_df));
rownames(omics_validation_df) <- as.vector(unlist(omics_validation_df1[1,]))
# Since the 2nd row contains response/class/status inormation we want to use that as a response variable
colnames(omics_validation_df)[1] <- "response"


randomforest_analysis <- function(omics_train_df, omics_validation_df, output_folder){
    ## Build a Random Forest model
     rf_model = randomForest(response ~ ., xtest=omics_validation_df[,2:ncol(omics_validation_df)], 
                              ytest=omics_validation_df$response, data=omics_train_df,
                              ntree = 1000, proximity = TRUE)

    pdf(paste(output_folder,"/all_random_forest_single_omics_results.pdf",sep=""))
    varImpPlot(rf_model, main="Variable Importances in the Validation Data")
    MDSplot(rf_model, omics_train_df$response,pch=as.numeric(omics_train_df$response), main="MDS (PCA-like) proximity plot of samples")
    plot(rf_model, main="Error rate per number of trees in the Random Forest model")
    plot(outlier(rf_model, cls=rf_model$y),type='h',col=as.numeric(rf_model$y),main="Outliers in the Validation Data\nbased on the proximity score computed by RF", ylab="Normalized score", xlab="Sample Index")
    dev.off()

    rf_model_results <- cbind(as.character(rf_model$y),as.character(rf_model$predicted))
    colnames(rf_model_results) <- c("validation_actual", "validation_predicted")
    write.table(file=paste(output_folder,"/random_forest_actual_predicted.tsv",sep=""),rf_model_results,sep="\t", row.names=F)
    write.table(file=paste(output_folder,"/random_forest_confusion_matrix.tsv",sep=""), rf_model$confusion,col.names = NA,sep="\t")
}

pca_analysis <- function(omics_train_df, omics_validation_df, output_folder){
    pdf(paste(output_folder,"/all_pca_single_omics_results.pdf",sep=""))
    omics_train1 <- as.data.frame(lapply(omics_train_df[,2:ncol(omics_train_df)],function(x){as.double(as.character(x))}));
    rownames(omics_train1) <- rownames(omics_train_df);
    omics_valid1 <- as.data.frame(lapply(omics_validation_df[,2:ncol(omics_validation_df)],function(x){as.double(as.character(x))}));
    rownames(omics_valid1) <- rownames(omics_validation_df);
    
    # Currently PCA tuning only shows how many components are necessary, but we don't use that information and use the default of 2 components
    tune.pca(omics_train1)
    train_pca <- pca(omics_train1)
    plotIndiv(train_pca,group=omics_train_df$response, legend = TRUE,title="PCA on Training Data", legend.title="Class")

    tune.pca(omics_valid1);
    validation_pca <- pca(omics_valid1)
    plotIndiv(validation_pca,group=omics_validation_df$response, legend = TRUE,title="PCA on Validation Data", legend.title="Class")
    
    plotLoadings(train_pca,group=omics_train_df$response, legend = TRUE, title="PCA Loadings on Training Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(train_pca,group=omics_train_df$response, legend = TRUE, title="PCA Loadings on Training Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotLoadings(validation_pca,group=omics_validation_df$response, legend = TRUE, title="PCA Loadings on Validation Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(validation_pca,group=omics_validation_df$response, legend = TRUE, title="PCA Loadings on Validation Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotVar(train_pca,cutoff=0.8,var.names=F, title="Training Data: Features with Correlations > 80%")
    plotVar(validation_pca,cutoff=0.8,var.names=F, title="Validation Data: Features with Correlations > 80%")

    dev.off()
}

sPCA_analysis <- function(omics_train_df, omics_validation_df, output_folder, comp1_feature_count=100,comp2_feature_count=100){
    pdf(paste(output_folder,"/all_sPCA_single_omics_results.pdf",sep=""))
    omics_train1 <- as.data.frame(lapply(omics_train_df[,2:ncol(omics_train_df)],function(x){as.double(as.character(x))}));
    rownames(omics_train1) <- rownames(omics_train_df);
    omics_valid1 <- as.data.frame(lapply(omics_validation_df[,2:ncol(omics_validation_df)],function(x){as.double(as.character(x))}));
    rownames(omics_valid1) <- rownames(omics_validation_df);

    train_spca <- spca(omics_train1,keepX = c(comp1_feature_count,comp2_feature_count), scale = TRUE)
    plotIndiv(train_spca, group=omics_train_df$response, legend = TRUE, title="sPCA on Training Data", legend.title="Class")

    validation_spca <- spca(omics_valid1,keepX = c(comp1_feature_count,comp2_feature_count), scale = TRUE)
    plotIndiv(validation_spca, group=omics_validation_df$response, legend = TRUE, title="sPCA on Validation Data", legend.title="Class")
    
    plotLoadings(train_spca,group=omics_train_df$response, legend = TRUE, title="sPCA Loadings on Training Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(train_spca,group=omics_train_df$response, legend = TRUE, title="sPCA Loadings on Training Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotLoadings(validation_spca,group=omics_validation_df$response, legend = TRUE, title="sPCA Loadings on Validation Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(validation_spca,group=omics_validation_df$response, legend = TRUE, title="sPCA Loadings on Validation Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotVar(train_spca,cutoff=0.8,var.names=F, title="Training Data: Features with Correlations > 80%")
    plotVar(validation_spca,cutoff=0.8,var.names=F, title="Validation Data: Features with Correlations > 80%")
    
    dev.off()

}

# Now starts classification methods with mixOmics package
plsda_analysis <- function(omics_train_df, omics_validation_df, output_folder){
    pdf(paste(output_folder,"/all_plsda_single_omics_results.pdf",sep=""))
    omics_train1 <- as.data.frame(lapply(omics_train_df[,2:ncol(omics_train_df)],function(x){as.double(as.character(x))}));
    rownames(omics_train1) <- rownames(omics_train_df);
    omics_valid1 <- as.data.frame(lapply(omics_validation_df[,2:ncol(omics_validation_df)],function(x){as.double(as.character(x))}));
    rownames(omics_valid1) <- rownames(omics_validation_df);

    # Tune for the best number of components and for each components using at least comp1_feature_count features
    set.seed(29)
    train_comp_count_plsda <- plsda(omics_train1, omics_train_df$response);
    tune_comp_plsda <- perf(train_comp_count_plsda, validation = "Mfold", folds=3, progressBar = F, nrepeat = 3);
    plot(tune_comp_plsda, legend.position = "horizontal", sd=T)
        
    train_plsda <- plsda(omics_train1, omics_train_df$response, ncomp=2)
    plotIndiv(train_plsda, group=omics_train_df$response, legend = TRUE, title="PLS-DA on Training Data", legend.title="Class")

    validation_plsda <- plsda(omics_valid1, omics_validation_df$response, ncomp=2)
    plotIndiv(validation_plsda, group=omics_validation_df$response, legend = TRUE, title="PLS-DA on Validation Data", legend.title="Class")
    
    
    plotLoadings(train_plsda,group=omics_train_df$response, legend = TRUE, title="PLS-DA Loadings on Training Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(train_plsda,group=omics_train_df$response, legend = TRUE, title="PLS-DA Loadings on Training Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotLoadings(validation_plsda,group=omics_validation_df$response, legend = TRUE, title="PLS-DA Loadings on Validation Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(validation_plsda,group=omics_validation_df$response, legend = TRUE, title="PLS-DA Loadings on Validation Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotVar(train_plsda,cutoff=0.8,var.names=F, title="Training Data: Features with Correlations > 80%")
    plotVar(validation_plsda,cutoff=0.8,var.names=F, title="Validation Data: Features with Correlations > 80%")

    cim(train_plsda, title="Heatmap of the Training data")
    cim(validation_plsda, title="Heatmap of the Validation data")
    
    auroc(train_plsda, newdata=omics_valid1, outcome.test = omics_validation_df$response, title="ROC Curve of the sPLS-DA on Validation Data\nComponent 1", roc.comp=1);

    auroc(train_plsda, newdata=omics_valid1, outcome.test = omics_validation_df$response, title="ROC Curve of the sPLS-DA on Validation Data\nComponent 2", roc.comp=2);


    dev.off()
}


# Sparse PLS-DA is a PLS-DA with feature selection capability
sPLSda_analysis <- function(omics_train_df, omics_validation_df, output_folder, comp1_feature_count=100,comp2_feature_count=100){
    pdf(paste(output_folder,"/all_sPLS_DA_single_omics_results.pdf",sep=""))
    omics_train1 <- as.data.frame(lapply(omics_train_df[,2:ncol(omics_train_df)],function(x){as.double(as.character(x))}));
    rownames(omics_train1) <- rownames(omics_train_df);
    omics_valid1 <- as.data.frame(lapply(omics_validation_df[,2:ncol(omics_validation_df)],function(x){as.double(as.character(x))}));
    rownames(omics_valid1) <- rownames(omics_validation_df);

    # Tune for the best number of components and for each components using at least comp1_feature_count features
    train_comp_count_splsda <- splsda(omics_train1, omics_train_df$response, keepX=rep(comp1_feature_count,10), ncomp=10);
    tune_comp_splsda <- perf(train_comp_count_splsda, validation = "Mfold", folds=3, progressBar = F, nrepeat = 3);
    plot(tune_comp_splsda, legend.position = "horizontal", sd=T)
    # Tune sPLS-DA for necessary number of components and features per component, while keeping component count at 2
    # Based on the previous figure you can decide on the number of components, but currently by default I am using 2 components
    set.seed(29)
    keepX_list <- c(5:10, seq(20, 100,10)); # Test different number of features
    tune_feature_count_splsda <- tune.splsda(omics_train1, omics_train_df$response, ncomp=2, validation="Mfold", folds = 3, dist="max.dist", progressBar=T, measure="BER", test.keepX=keepX_list, nrepeat=30);
    plot(tune_feature_count_splsda);#, color=color.jet(3));#, title="Feature Selection for Component 1 only & Comp 1 & 2")
        
    train_splsda <- splsda(omics_train1, omics_train_df$response, ncomp=2,keepX=tune_feature_count_splsda$choice.keepX)
    plotIndiv(train_splsda, group=omics_train_df$response, legend = TRUE, title="sPLS-DA on Training Data", legend.title="Class")

    validation_splsda <- splsda(omics_valid1, omics_validation_df$response, ncomp=2,keepX=tune_feature_count_splsda$choice.keepX)
    plotIndiv(validation_splsda, group=omics_validation_df$response, legend = TRUE, title="sPLS-DA on Validation Data", legend.title="Class")
    
    
    plotLoadings(train_splsda,group=omics_train_df$response, legend = TRUE, title="sPLS-DA Loadings on Training Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(train_splsda,group=omics_train_df$response, legend = TRUE, title="sPLS-DA Loadings on Training Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotLoadings(validation_splsda,group=omics_validation_df$response, legend = TRUE, title="sPLS-DA Loadings on Validation Data\nComponent 1", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=1)
    plotLoadings(validation_splsda,group=omics_validation_df$response, legend = TRUE, title="sPLS-DA Loadings on Validation Data\nComponent 2", legend.title="Class", method="median", ndisplay=20, size.name=0.5, size.legend=0.5, comp=2)
    
    plotVar(train_splsda,cutoff=0.8,var.names=F, title="Training Data: Features with Correlations > 80%")
    plotVar(validation_splsda,cutoff=0.8,var.names=F, title="Validation Data: Features with Correlations > 80%")

    cim(train_splsda, title="Heatmap of the Training data")
    cim(validation_splsda, title="Heatmap of the Validation data")
    
    auroc(train_splsda, newdata=omics_valid1, outcome.test = omics_validation_df$response, title="ROC Curve of the sPLS-DA on Validation Data\nComponent 1", roc.comp=1);

    auroc(train_splsda, newdata=omics_valid1, outcome.test = omics_validation_df$response, title="ROC Curve of the sPLS-DA on Validation Data\nComponent 2", roc.comp=2);

    dev.off()
}


# Call the relevant function per method
for(i in 1:length(methods)){
    switch(tolower(methods[i]),
           randomforest=randomforest_analysis(omics_train_df, omics_validation_df, output_folder),
           pca=pca_analysis(omics_train_df, omics_validation_df, output_folder),
           spca=sPCA_analysis(omics_train_df, omics_validation_df, output_folder),
           plsda=plsda_analysis(omics_train_df, omics_validation_df, output_folder),
           splsda=sPLSda_analysis(omics_train_df, omics_validation_df, output_folder)
    )
}
           
#     if(tolower(methods[i]) == "randomforest"){
#         randomforest_analysis(omics_train_df, omics_validation_df, output_folder);
#     } else if(tolower(methods[i])=="pca"){
#         pca_analysis(omics_train_df, omics_validation_df, output_folder);
#     }
